import 'package:flutter/widgets.dart';

import 'package:responsive_layout_builder/src/layout_size.dart';
import 'package:responsive_layout_builder/src/screen_size_settings.dart';

/// Signature for a function that builds a widget given an [LayoutSize].
///
/// Used by [ResponsiveLayoutBuilder.builder].
typedef Widget ResponsiveLayoutWidgetBuilder(
    BuildContext context, LayoutSize size);

/// Builds a widget tree that can depend on the parent widget's [LayoutSize] (orientation and layout).
///
/// Constraints are based on all available space that this widget can occupy, not on screen size
class ResponsiveLayoutBuilder extends StatefulWidget {
  /// Builds the widgets below this widget given this widget's [LayoutSize]
  final ResponsiveLayoutWidgetBuilder builder;

  /// Sets orientation usage
  ///
  /// Defaults to [true]
  ///
  /// If [false] orientation is ommited
  final bool useOrientation;

  /// Sets default size if no condition was passed
  final LayoutSize defaultSize;

  /// Sets screen sizes for different devices which affects on [LayoutSize]
  final ScreenSizeSettings screenSizeSettings;

  /// Creates an responsive layout builder.
  ///
  /// The [builder] argument must not be null.
  const ResponsiveLayoutBuilder({
    Key key,
    @required this.builder,
    this.useOrientation = true,
    this.defaultSize = LayoutSize.middleMobile,
    this.screenSizeSettings = const ScreenSizeSettings(),
  })  : assert(builder != null),
        assert(useOrientation != null),
        assert(screenSizeSettings != null),
        super(key: key);

  @override
  _ResponsiveLayoutBuilderState createState() =>
      _ResponsiveLayoutBuilderState();
}

class _ResponsiveLayoutBuilderState extends State<ResponsiveLayoutBuilder> {
  LayoutSize _getLayoutSizeWithOrientation(
      Orientation orientation, BoxConstraints constraints) {
    LayoutSize size;

    if (orientation == Orientation.portrait &&
        constraints.maxWidth <= widget.screenSizeSettings.smallMobile) {
      // Less then 360 Portrait
      size = LayoutSize.smallMobilePortrait;
    } else if (orientation == Orientation.portrait &&
        constraints.maxWidth > widget.screenSizeSettings.smallMobile &&
        constraints.maxWidth <= widget.screenSizeSettings.middleMobile) {
      // Greater then 360 and Less then 600 Portrait
      size = LayoutSize.middleMobilePortrait;
    } else if (orientation == Orientation.landscape &&
        constraints.maxHeight <= widget.screenSizeSettings.smallMobile) {
      // Less then 360 Landscape
      size = LayoutSize.smallMobileLandscape;
    } else if (orientation == Orientation.landscape &&
        constraints.maxHeight > widget.screenSizeSettings.smallMobile &&
        constraints.maxHeight <= widget.screenSizeSettings.middleMobile) {
      // Greater then 360 and Less then 600 Landscape
      size = LayoutSize.middleMobileLandscape;
    }

    return size;
  }

  LayoutSize _getLayoutSizeWithoutOrientation(BoxConstraints constraints) {
    LayoutSize size;

    if (constraints.biggest.shortestSide <=
        widget.screenSizeSettings.smallMobile) {
      // Less then 360
      size = LayoutSize.smallMobile;
    } else if (constraints.biggest.shortestSide >
            widget.screenSizeSettings.smallMobile &&
        constraints.biggest.shortestSide <=
            widget.screenSizeSettings.middleMobile) {
      // Greater then 360 and Less then 600
      size = LayoutSize.middleMobile;
    }

    return size;
  }

  LayoutSize _getLayoutSize(
      Orientation orientation, BoxConstraints constraints) {
    LayoutSize size;

    if (constraints.biggest.shortestSide <= widget.screenSizeSettings.watch) {
      // Less then 220
      size = LayoutSize.watch;
    } else if (constraints.biggest.shortestSide >
            widget.screenSizeSettings.watch &&
        constraints.biggest.shortestSide <=
            widget.screenSizeSettings.middleMobile) {
      // Greater then 220 and Less then 600
      size = (widget.useOrientation
              ? _getLayoutSizeWithOrientation(orientation, constraints)
              : _getLayoutSizeWithoutOrientation(constraints)) ??
          widget.defaultSize ??
          LayoutSize.middleMobile;
    } else if (constraints.biggest.shortestSide >
        widget.screenSizeSettings.middleMobile) {
      // Greater then 600
      size = LayoutSize.tabletOrDesktop;
    } else {
      // If no condition was satisfied
      size = widget.defaultSize ?? LayoutSize.middleMobile;
    }

    return size;
  }

  @override
  Widget build(BuildContext context) {
    return OrientationBuilder(
      key: widget.key,
      builder: (context, orientation) {
        return LayoutBuilder(
          builder: (context, constraints) {
            return widget.builder(
              context,
              _getLayoutSize(orientation, constraints),
            );
          },
        );
      },
    );
  }
}
